from pyparsing import (
    CaselessKeyword,
    Combine,
    Keyword,
    LineEnd,
    LineStart,
    Literal,
    NotAny,
    Optional,
    Word,
    ZeroOrMore,
    alphanums,
    nums,
    pyparsing_common,
)


def set_default_scheme(toks):
    if "scheme" not in toks:
        toks["scheme"] = "ssh"
    return toks


def scheme(s):
    return (
        CaselessKeyword(s)
        .setParseAction(pyparsing_common.downcaseTokens)
        .setResultsName("scheme")
    )


PART = Word(alphanums)
LABEL = PART + ZeroOrMore("-" + PART)
HOSTNAME = LABEL + ZeroOrMore("." + LABEL)
HOST = (
    Combine(HOSTNAME)
    .setParseAction(pyparsing_common.downcaseTokens)
    .setResultsName("host")
)
PORT = (
    Word(nums).setParseAction(lambda toks: int(toks[0])).setResultsName("port")
)
USER = Word(alphanums, alphanums + "_-").setResultsName("user")
PATH = Word(alphanums + "+,/._-~").setResultsName("path")

LOCAL_URL = ((Keyword("file").setResultsName("scheme") + "://") | "/") + PATH

HTTP_URL = (
    (scheme("http") | scheme("https")).setResultsName("scheme")
    + "://"
    + Optional(USER + Literal("@"))
    + HOST
    + Optional(Literal(":") + PORT)
    + "/"
    + PATH
)

SSH_URL = (
    scheme("ssh")
    + "://"
    + Optional(USER + "@")
    + HOST
    + Optional(Literal(":") + PORT)
    + Literal("/")
    + PATH
)
SCP_LIKE_SSH_URL = (
    Optional(USER + "@") + HOST + NotAny(Literal("://")) + Literal(":") + PATH
).setParseAction(set_default_scheme)
GIT_URL = (
    scheme("git")
    + "://"
    + Optional(USER + "@")
    + HOST
    + Optional(Literal(":") + PORT)
    + Literal("/")
    + PATH
)
URL = (
    LineStart() + (HTTP_URL | SSH_URL | GIT_URL | SCP_LIKE_SSH_URL) + LineEnd()
)



import pytest

from giturl import GitURL, InvalidURL


TESTDATA_GITURL_VALID = [
    (
        "https://github.com/kinkerl/glowing-octo-succotash.git",
        GitURL(
            scheme="https",
            host="github.com",
            path="kinkerl/glowing-octo-succotash.git",
        ),
    ),
    (
        "https://git@github.com/kinkerl/glowing-octo-succotash.git",
        GitURL(
            scheme="https",
            user="git",
            host="github.com",
            path="kinkerl/glowing-octo-succotash.git",
        ),
    ),
    (
        "https://git@github.com:8443/kinkerl/glowing-octo-succotash.git",
        GitURL(
            scheme="https",
            user="git",
            host="github.com",
            path="kinkerl/glowing-octo-succotash.git",
            port=8443,
        ),
    ),
    (
        "http://github.com/kinkerl/glowing-octo-succotash.git",
        GitURL(
            scheme="http",
            host="github.com",
            path="kinkerl/glowing-octo-succotash.git",
        ),
    ),
    (
        "git://github.com/kinkerl/glowing-octo-succotash.git",
        GitURL(
            scheme="git",
            host="github.com",
            path="kinkerl/glowing-octo-succotash.git",
        ),
    ),
    (
        "git://git.divio.com/ci-test-project-do-not-delete.git",
        GitURL(
            scheme="git",
            host="git.divio.com",
            path="ci-test-project-do-not-delete.git",
        ),
    ),
    (
        "ssh://github.com/kinkerl/glowing-octo-succotash.git",
        GitURL(
            scheme="ssh",
            host="github.com",
            path="kinkerl/glowing-octo-succotash.git",
        ),
    ),
    (
        "ssh://git@github.com/kinkerl/glowing-octo-succotash.git",
        GitURL(
            scheme="ssh",
            user="git",
            host="github.com",
            path="kinkerl/glowing-octo-succotash.git",
        ),
    ),
    (
        "ssh://git.divio.com/ci-test-project-do-not-delete.git",
        GitURL(
            scheme="ssh",
            host="git.divio.com",
            path="ci-test-project-do-not-delete.git",
        ),
    ),
    (
        "ssh://git.divio.com:2222/ci-test-project-do-not-delete.git",
        GitURL(
            scheme="ssh",
            host="git.divio.com",
            path="ci-test-project-do-not-delete.git",
            port=2222,
        ),
    ),
    (
        "SSH://GIT.divio.com:2222/ci-test-project-do-not-delete.git",
        GitURL(
            scheme="ssh",
            host="git.divio.com",
            path="ci-test-project-do-not-delete.git",
            port=2222,
        ),
    ),
    (
        "git@github.com:vxsx/animated-octo-doodle.git",
        GitURL(
            scheme="ssh",
            user="git",
            host="github.com",
            path="vxsx/animated-octo-doodle.git",
        ),
    ),
]
TESTDATA_GITURL_INVALID = [
    # Invalid scheme
    "nothing://github.com/kinkerl/glowing-octo-succotash.git",
    # : is not correct
    "ssh://github.com:kinkerl/glowing-octo-succotash.git",
    # : is not correct
    "git://github.com:kinkerl/glowing-octo-succotash.git",
    # Spaces
    "git@github.com:vxsx/animated-octo-d oodle.git",
    " git@github.com:vxsx/animated-octo-d oodle.git",
    "git@github.com:vxsx/animated-octo-d oodle.git ",
    "git @github.com:vxsx/animated-octo-d oodle.git",
    "git@github.co m:vxsx/animated-octo-d oodle.git",
    "git@github.com:vxsx/ animated-octo-d oodle.git",
    "",
    " ",
    "ssh://",
    "ssh://a",
]

TESTDATA_GITURL_NORMALIZE = [
    (
        "https://github.com/kinkerl/glowing-octo-succotash.git",
        "https://github.com/kinkerl/glowing-octo-succotash.git",
    ),
    (
        "git://github.com/kinkerl/glowing-octo-succotash.git",
        "git://github.com/kinkerl/glowing-octo-succotash.git",
    ),
    (
        "git@github.com:kinkerl/glowing-octo-succotash.git",
        "ssh://git@github.com/kinkerl/glowing-octo-succotash.git",
    ),
    (
        "git://git.divio.com/ci-test-project-do-not-delete.git",
        "git://git.divio.com/ci-test-project-do-not-delete.git",
    ),
    (
        "ssh://github.com/kinkerl/glowing-octo-succotash.git",
        "ssh://github.com/kinkerl/glowing-octo-succotash.git",
    ),
    (
        "ssh://git.divio.com/ci-test-project-do-not-delete.git",
        "ssh://git.divio.com/ci-test-project-do-not-delete.git",
    ),
    (
        "ssh://git.DiVIO.com/ci-test-project-do-not-delete.git",
        "ssh://git.divio.com/ci-test-project-do-not-delete.git",
    ),
    (
        "SSH://git.DiVIO.com/ci-test-project-do-not-delete.git",
        "ssh://git.divio.com/ci-test-project-do-not-delete.git",
    ),
]


@pytest.mark.parametrize("url,expected", TESTDATA_GITURL_VALID)
def test_parse_valid(url, expected):
    parsed = GitURL.parse(url)
    assert parsed == expected


@pytest.mark.parametrize("url", TESTDATA_GITURL_INVALID)
def test_parse_invalid(url):
    with pytest.raises(InvalidURL):
        GitURL.parse(url)


# url styles we do not support
@pytest.mark.parametrize("url,expected", TESTDATA_GITURL_NORMALIZE)
def test_normalize(url, expected):
    assert str(GitURL.parse(url)) == expected



import attr
from pyparsing import ParseException

from .exceptions import InvalidURL
from .schema import URL

__author__ = "Swiss Data Science Center"
__email__ = "contact@datascience.ch"
__version__ = "0.1.0"
__url__ = "https://datascience.ch/"


@attr.s(frozen=True)
class GitURL:
    scheme = attr.ib(default=None)
    host = attr.ib(default=None)
    path = attr.ib(default=None)
    user = attr.ib(default=None)
    port = attr.ib(default=None)
    _parsing_result = attr.ib(default=None, cmp=False, repr=False)

    def __str__(self):
        authinfo = f"{self.user}@" if self.user else ""
        netloc = f"{self.host}:{self.port}" if self.port else self.host
        return f"{self.scheme}://{authinfo}{netloc}/{self.path}"

    @classmethod
    def parse(cls, url):
        try:
            res = URL.parseString(url, parseAll=True)
        except ParseException:
            raise InvalidURL(url)
        return GitURL(**res, parsing_result=res)


def parse(url):
    """
    Handy alias to
    """
    return GitURL.parse(url)



#!/usr/bin/env python

import io
import os
import re
import sys

from setuptools import find_packages, setup


PACKAGE = "yagup"


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist bdist_wheel upload")
    sys.exit()


class Setup(object):
    @staticmethod
    def read(fname, fail_silently=False):
        """
        Read the content of the given file. The path is evaluated from the
        directory containing this file.
        """
        try:
            filepath = os.path.join(os.path.dirname(__file__), fname)
            with io.open(filepath, "rt", encoding="utf8") as f:
                return f.read()
        except:
            if not fail_silently:
                raise
            return ""

    @classmethod
    def requirements(cls, fname):
        """
        Create a list of requirements from the output of the pip freeze command
        saved in a text file.
        """
        packages = cls.read(fname, fail_silently=True).split("\n")
        packages = (p.strip() for p in packages)
        packages = (p for p in packages if p and not p.startswith("#"))
        packages = (p for p in packages if p and not p.startswith("https://"))
        return list(packages)

    @staticmethod
    def get_files(*bases):
        """
        List all files in a data directory.
        """
        for base in bases:
            basedir, _ = base.split(".", 1)
            base = os.path.join(os.path.dirname(__file__), *base.split("."))

            rem = len(os.path.dirname(base)) + len(basedir) + 2

            for root, dirs, files in os.walk(base):
                for name in files:
                    yield os.path.join(basedir, root, name)[rem:]

    @classmethod
    def meta(cls, key):
        data = cls.read(os.path.join(PACKAGE, "__init__.py"))
        version = (
            re.search(u"__{}__\s*=\s*u?('|\")([^'\"]+)('|\")".format(key), data)
            .group(2)
            .strip()
        )
        return version

    @classmethod
    def author(cls):
        return cls.meta("author")

    @classmethod
    def email(cls):
        return cls.meta("email")

    @classmethod
    def version(cls):
        return cls.meta("version")

    @classmethod
    def url(cls):
        return cls.meta("url")

    @classmethod
    def longdesc(cls):
        return cls.read("README.md")

    @staticmethod
    def test_links():
        # Test if hardlinks work. This is a workaround until
        # http://bugs.python.org/issue8876 is solved
        if hasattr(os, "link"):
            tempfile = __file__ + ".tmp"
            try:
                os.link(__file__, tempfile)
            except OSError as e:
                if e.errno == 1:  # Operation not permitted
                    del os.link
                else:
                    raise
            finally:
                if os.path.exists(tempfile):
                    os.remove(tempfile)


Setup.test_links()

setup(
    name=PACKAGE,
    version=Setup.version(),
    author=Setup.author(),
    author_email=Setup.email(),
    include_package_data=True,
    zip_safe=False,
    license="BSD",
    packages=find_packages(),
    package_dir={PACKAGE: PACKAGE},
    description="Parsing and validating git urls.",
    install_requires=Setup.requirements("requirements.txt"),
    long_description=Setup.longdesc(),
    long_description_content_type="text/markdown",
    url="https://github.com/SwissDataScienceCenter/yagup",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
